<?php

namespace FiloBlu\Refilo\Model\Indexer\Core;

use FiloBlu\Refilo\Model\Adapter\Elastic;
use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterfaceFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\StoreManagerInterface;

class ElasticWebsiteProcessor extends BaseElasticIndexer
{
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * BaseWebsiteProcessor constructor.
     * @param SqlToMongoAdapterInterface $sqlToMongoAdapter
     * @param QueryProviderInterfaceFactory $queryProviderFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     * @param Elastic $elastic
     * @param string $baseKey
     * @param string $baseCollection
     * @param bool $insertMode
     * @param string $databaseConfigurationKey
     */
    public function __construct(
        SqlToMongoAdapterInterface $sqlToMongoAdapter,
        QueryProviderInterfaceFactory $queryProviderFactory,
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager,
        Elastic $elastic,
        string $baseKey,
        string $baseCollection,
        bool $insertMode,
        string $databaseConfigurationKey
    ) {
        parent::__construct(
            $sqlToMongoAdapter,
            $queryProviderFactory,
            $scopeConfig,
            $elastic,
            $baseKey,
            $baseCollection,
            $insertMode,
            $databaseConfigurationKey
        );
        $this->storeManager = $storeManager;
    }

    protected function performAction($ids = null)
    {
        foreach ($this->storeManager->getWebsites() as $website) {
            /** @var QueryProviderInterface $provider */
            $provider = $this->queryProviderFactory->create();
            if ($ids) {
                $provider->filter($ids);
            }
            $provider->website($website->getId());
            $result = $this->sqlToMongoAdapter->adaptFromSql($provider);
            foreach ($website->getStores() as $store) {

                if (!$store->getIsActive())
                    continue;

                $index = sprintf(
                    $this->getTargetMongoCollection(),
                    $store->getCode()
                );

                if (!$this->getInsertMode() && !$this->elastic->checkIndex($index)) {
                    continue;
                }

                $this->elastic->update(
                    $this->getBaseKey(),
                    $result,
                    $index,
                    $this->getInsertMode(),
                    $this->getMongoDb()
                );

                //$this->elastic->checkConsistency($index);

            }
        }
    }
}
