<?php


namespace FiloBlu\Refilo\Model\Indexer\Processor\Type;

use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;

class Configurable extends Simple
{
    /**
     * @var ProductRepository
     */
    private $productRepository;
    /**
     * @var FilterGroup
     */
    private $filterGroup;
    /**
     * @var FilterBuilder
     */
    private $filterBuilder;
    /**
     * @var Status
     */
    private $productStatus;
    /**
     * @var Simple
     */
    private $_simple_product;

    /**
     * @var array
     */
    private $child_products;
    /**
     * @var SearchCriteriaInterface
     */
    private $criteria;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * Configurable constructor.
     * @param ProductRepository $productRepository
     * @param FilterGroup $filterGroup
     * @param FilterBuilder $filterBuilder
     * @param Status $productStatus
     * @param Simple $simple_product
     * @param SearchCriteriaInterface $criteria
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        ProductRepository $productRepository,
        FilterGroup $filterGroup,
        FilterBuilder $filterBuilder,
        Status $productStatus,
        Simple $simple_product,
        SearchCriteriaInterface $criteria,
        SearchCriteriaBuilder $searchCriteriaBuilder
    )
    {

        $this->productRepository = $productRepository;
        $this->filterGroup = $filterGroup;
        $this->filterBuilder = $filterBuilder;
        $this->productStatus = $productStatus;
        $this->_simple_product = $simple_product;
        $this->criteria = $criteria;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    public function process()
    {
        parent::process();
        $this->getChilds();
        $this->getVariants();
        return $this->_item;

    }

    protected function getChilds() {

        /**
         * @var Magento\Catalog\Api\Data\ProductExtension
         */
        $extension_attributes =  $this->_product->getData("extension_attributes");

        $this->_item->childs = [];
        $this->_item->childs = array_map('intval', array_keys($extension_attributes->getConfigurableProductLinks()));

//        if (count($this->_item->childs))
//            $this->child_products = $this->processChilds($this->_item->childs);

    }

    /**
     * @return array
     */
    public function getChildProducts()
    {
        return $this->child_products;
    }

    protected function processChilds($ids) {

        $searchCriteria = $this->searchCriteriaBuilder
            ->addFilter('entity_id',$ids,'in')
            ->addFilter('status',$this->productStatus->getVisibleStatusIds(),'in')
            ->create();

        $productItems = $this->productRepository->getList($searchCriteria)->getItems();

        $items = [];

        foreach ($productItems as $product) {
            $this->_simple_product->setProduct($product);
            $item = $this->_simple_product->process();
            $this->_item->filters = (object) array_merge((array) $this->_item->filters, (array) $item->filters);

            $items[] = $item;

        }
        $this->productRepository->cleanCache();
        unset($productItems);
        return $items;

    }

    protected function getVariants() {

        /**
         * @var Magento\Catalog\Api\Data\ProductExtension
         */
        $extension_attributes =  $this->_product->getData("extension_attributes");

        $this->_item->variants = new \stdClass();

        $variants = $extension_attributes->getConfigurableProductOptions();

        if (count($variants) && $this->_product->getData('type_id') == 'configurable') {
            $this->_item->variants = $this->buildConfigurableVariants($variants);
        }

    }

    public function buildConfigurableVariants($variants) {

        $variants_parsed = [];

        foreach ($variants as $variant) {

            $variant_parsed = new \stdClass();
            $sub_filter = new \stdClass();

            $variant_parsed->id = $variant->getId();
            $variant_parsed->label = $variant->getProductAttribute()->getStoreLabel();
            $variant_parsed->use_default = (int)$variant->getIsUseDefault();
            $variant_parsed->position = $variant->getPosition();
            $variant_parsed->attribute_id = $variant->getAttributeId();
            $variant_parsed->attribute_code = $variant->getProductAttribute()->getAttributeCode();
            $variant_parsed->values = $variant->getOptions();

            $additional_data = json_decode($variant->getProductAttribute()->getData('additional_data'));
            $variant_parsed->type = $variant->getProductAttribute()->getData('frontend_input');
            if ($additional_data)
                $variant_parsed->rendered = property_exists($additional_data, "swatch_input_type") ? $additional_data->swatch_input_type : '';

            $labels = [];
            $variant_parsed->options = [];

            foreach ($variant_parsed->values as $value) {
                $variant_parsed->options[] = (int)$value["value_index"];
                $labels[] = $value["store_label"];
                //$labels_aggs[] = ["key"=> (int)$value["value_index"], "value" => $value["store_label"]] ;

            }

            $sub_filter->title = $variant_parsed->label;
            $sub_filter->code = $variant_parsed->attribute_code;
            $sub_filter->type = $variant->getFontendInput();
            $sub_filter->key = $variant_parsed->options;
            $sub_filter->value = $labels;
            //$sub_filter->aggs = $labels_aggs;

            $variants_parsed[] = $variant_parsed;
            $this->_item->filters->{$variant_parsed->attribute_code} = $sub_filter;

        }
        return $variants_parsed;

    }

}
